@@ -19,13 +19,13 @@ use {
             check_sysvar_keyed_account, from_keyed_account, get_signers, keyed_account_at_index,
             KeyedAccount,
         },
-        process_instruction::{get_sysvar, InvokeContext},
+        process_instruction::InvokeContext,
         program_utils::limited_deserialize,
         pubkey::Pubkey,
         system_instruction,
-        sysvar::{self, clock::Clock, rent::Rent, slot_hashes::SlotHashes, Sysvar},
+        sysvar::{self, clock::Clock, rent::Rent, slot_hashes::SlotHashes},
     },
-    std::collections::HashSet,
+    std::{collections::HashSet, sync::Arc},
     thiserror::Error,
 };
 
@@ -311,17 +311,37 @@ fn verify_rent_exemption(
     }
 }
 
-/// This method facilitates a transition from fetching sysvars from keyed
+/// These methods facilitate a transition from fetching sysvars from keyed
 /// accounts to fetching from the sysvar cache without breaking consensus. In
-/// order to keep consistent behavior, it continues to enforce the same checks
+/// order to keep consistent behavior, they continue to enforce the same checks
 /// as `solana_sdk::keyed_account::from_keyed_account` despite dynamically
 /// loading them instead of deserializing from account data.
-fn get_sysvar_with_keyed_account_check<S: Sysvar>(
-    keyed_account: &KeyedAccount,
-    invoke_context: &dyn InvokeContext,
-) -> Result<S, InstructionError> {
-    check_sysvar_keyed_account::<S>(keyed_account)?;
-    get_sysvar(invoke_context, keyed_account.unsigned_key())
+mod get_sysvar_with_keyed_account_check {
+    use super::*;
+
+    pub fn clock(
+        keyed_account: &KeyedAccount,
+        invoke_context: &dyn InvokeContext,
+    ) -> Result<Arc<Clock>, InstructionError> {
+        check_sysvar_keyed_account::<Clock>(keyed_account)?;
+        invoke_context.get_sysvar_cache().get_clock()
+    }
+
+    pub fn rent(
+        keyed_account: &KeyedAccount,
+        invoke_context: &dyn InvokeContext,
+    ) -> Result<Arc<Rent>, InstructionError> {
+        check_sysvar_keyed_account::<Rent>(keyed_account)?;
+        invoke_context.get_sysvar_cache().get_rent()
+    }
+
+    pub fn slot_hashes(
+        keyed_account: &KeyedAccount,
+        invoke_context: &dyn InvokeContext,
+    ) -> Result<Arc<SlotHashes>, InstructionError> {
+        check_sysvar_keyed_account::<SlotHashes>(keyed_account)?;
+        invoke_context.get_sysvar_cache().get_slot_hashes()
+    }
 }
 
 pub fn process_instruction(
@@ -346,19 +366,19 @@ pub fn process_instruction(
 
     match limited_deserialize(data)? {
         VoteInstruction::InitializeAccount(vote_init) => {
-            let rent: Rent = get_sysvar_with_keyed_account_check(
+            let rent = get_sysvar_with_keyed_account_check::rent(
                 keyed_account_at_index(keyed_accounts, 1)?,
                 invoke_context,
             )?;
             verify_rent_exemption(me, &rent)?;
-            let clock: Clock = get_sysvar_with_keyed_account_check(
+            let clock = get_sysvar_with_keyed_account_check::clock(
                 keyed_account_at_index(keyed_accounts, 2)?,
                 invoke_context,
             )?;
             vote_state::initialize_account(me, &vote_init, &signers, &clock)
         }
         VoteInstruction::Authorize(voter_pubkey, vote_authorize) => {
-            let clock: Clock = get_sysvar_with_keyed_account_check(
+            let clock = get_sysvar_with_keyed_account_check::clock(
                 keyed_account_at_index(keyed_accounts, 1)?,
                 invoke_context,
             )?;
@@ -374,11 +394,11 @@ pub fn process_instruction(
         }
         VoteInstruction::Vote(vote) | VoteInstruction::VoteSwitch(vote, _) => {
             inc_new_counter_info!("vote-native", 1);
-            let slot_hashes: SlotHashes = get_sysvar_with_keyed_account_check(
+            let slot_hashes = get_sysvar_with_keyed_account_check::slot_hashes(
                 keyed_account_at_index(keyed_accounts, 1)?,
                 invoke_context,
             )?;
-            let clock: Clock = get_sysvar_with_keyed_account_check(
+            let clock = get_sysvar_with_keyed_account_check::clock(
                 keyed_account_at_index(keyed_accounts, 2)?,
                 invoke_context,
             )?;
@@ -389,11 +409,25 @@ pub fn process_instruction(
             let rent_sysvar = if invoke_context
                 .is_feature_active(&feature_set::reject_non_rent_exempt_vote_withdraws::id())
             {
-                Some(get_sysvar(invoke_context, &sysvar::rent::id())?)
+                Some(invoke_context.get_sysvar_cache().get_rent()?)
             } else {
                 None
             };
-            vote_state::withdraw(me, lamports, to, &signers, rent_sysvar)
+            let clock_if_feature_active = if invoke_context.is_feature_active(
+                &feature_set::reject_vote_account_close_unless_zero_credit_epoch::id(),
+            ) {
+                Some(invoke_context.get_sysvar_cache().get_clock()?)
+            } else {
+                None
+            };
+            vote_state::withdraw(
+                me,
+                lamports,
+                to,
+                &signers,
+                rent_sysvar.as_deref(),
+                clock_if_feature_active.as_deref(),
+            )
         }
         VoteInstruction::AuthorizeChecked(vote_authorize) => {
             if invoke_context.is_feature_active(&feature_set::vote_stake_checked_instructions::id())
@@ -484,18 +518,9 @@ mod tests {
                 .collect();
             let mut invoke_context = MockInvokeContext::new(keyed_accounts);
             let mut sysvar_cache = SysvarCache::default();
-            sysvar_cache.push_entry(
-                sysvar::rent::id(),
-                bincode::serialize(&Rent::free()).unwrap(),
-            );
-            sysvar_cache.push_entry(
-                sysvar::clock::id(),
-                bincode::serialize(&Clock::default()).unwrap(),
-            );
-            sysvar_cache.push_entry(
-                sysvar::slot_hashes::id(),
-                bincode::serialize(&SlotHashes::default()).unwrap(),
-            );
+            sysvar_cache.set_rent(Rent::free());
+            sysvar_cache.set_clock(Clock::default());
+            sysvar_cache.set_slot_hashes(SlotHashes::default());
             invoke_context.sysvar_cache = Cow::Owned(sysvar_cache);
             super::process_instruction(&Pubkey::default(), &instruction.data, &mut invoke_context)
         }
